@@ -16,7 +16,7 @@
 
 #include <stdio.h>
 
-namespace acommon {
+namespace aspell {
 
   class Config;
   class Speller;
@@ -31,34 +31,44 @@ namespace acommon {
     void clear();
     void reset();
     void process(FilterChar * & start, FilterChar * & stop);
-    void add_filter(IndividualFilter * filter);
-    // setup the filter where the string list is the list of 
-    // filters to use.
+    void add_filter(IndividualFilter * filter, bool own = true);
+
+    // 
+    Filter * shallow_copy();
+
     Filter();
     ~Filter();
 
- private:
+  private:
+
     typedef Vector<IndividualFilter *> Filters;
     Filters filters_;
+    Filters own_;
+
+  public:
+
+    typedef Filters::const_iterator Iterator;
+    Iterator begin() const {return filters_.begin();}
+    Iterator end()   const {return filters_.end();}
+
   };
 
   PosibErr<void> set_mode_from_extension(Config * config,
                                          ParmString filename,
                                          FILE * in = NULL);
-  
+ 
   PosibErr<void> setup_filter(Filter &, Config *, 
 			      bool use_decoder, 
 			      bool use_filter, 
 			      bool use_encoder);
   void activate_dynamic_filteroptions(Config *c);
   void activate_filter_modes(Config * config);
-  PosibErr<void> print_mode_help(const Config *, OStream &);
 
   void load_all_filters(Config * config);
 
-  PosibErr<bool> verify_version(const char * relOp, const char * actual,
-                                const char * required);
-  PosibErr<void> check_version(char * requirement);
+  PosibErr<bool> verify_version(const char * relOp, 
+                                const char * actual, const char * required);
+  PosibErr<void> check_version(const char * requirement);
 
   PosibErr<StringPairEnumeration *> available_filters(Config *);
   PosibErr<StringPairEnumeration *> available_filter_modes(Config *);
