@@ -81,9 +81,10 @@ struct AffEntry
 {
   const char *   appnd;
   const char *   strip;
+  const char *   flags;
   byte           appndl;
   byte           stripl;
-  byte           xpflg;
+  byte           xpflgs;
   char           achar;
   const Conds *  conds;
   //unsigned int numconds;
@@ -103,7 +104,7 @@ struct PfxEntry : public AffEntry
   bool check(const LookupInfo &, const AffixMgr * pmyMgr,
              ParmString, IntrCheckInfo &, GuessInfo *, bool cross = true) const;
 
-  inline bool          allow_cross() const { return ((xpflg & XPRODUCT) != 0); }
+  inline bool          allow_cross() const { return ((xpflgs & XPRODUCT) != 0); }
   inline byte flag() const { return achar;  }
   inline const char *  key() const  { return appnd;  }
   bool applicable(SimpleString) const;
@@ -123,12 +124,14 @@ struct SfxEntry : public AffEntry
 
   SfxEntry() {}
 
-  bool check(const LookupInfo &, ParmString, IntrCheckInfo &, GuessInfo *,
-             int optflags, AffEntry * ppfx);
+  bool check(const LookupInfo &, const AffixMgr * pmyMgr,
+             ParmString, IntrCheckInfo &, GuessInfo *,
+             const PfxEntry * ppfx, const SfxEntry * psfx) const;
 
-  inline bool          allow_cross() const { return ((xpflg & XPRODUCT) != 0); }
+  inline bool          allow_cross() const { return ((xpflgs & XPRODUCT) != 0); }
   inline byte flag() const { return achar;  }
   inline const char *  key() const  { return rappnd; } 
+  void add_outer_sfx_info(IntrCheckInfo *) const;
   bool applicable(SimpleString) const;
   SimpleString add(SimpleString, ObjStack & buf, int limit, SimpleString) const;
 };
@@ -269,6 +272,9 @@ static inline void max_(int & lhs, int rhs)
 // read in aff file and build up prefix and suffix entry objects 
 PosibErr<void> AffixMgr::parse_file(const char * affpath, Conv & iconv)
 {
+  //
+  two_fold_suffix = false;
+
   // io buffers
   String buf; DataPair dp;
 
@@ -285,6 +291,7 @@ PosibErr<void> AffixMgr::parse_file(const char * affpath, Conv & iconv)
   // read in each line ignoring any that do not
   // start with a known line type indicator
 
+  char circumfix_flag = '\0'; 
   char prev_aff = '\0';
 
   while (getdata_pair(afflst,dp,buf)) {
@@ -299,6 +306,12 @@ PosibErr<void> AffixMgr::parse_file(const char * affpath, Conv & iconv)
         return make_err(incorrect_encoding, affix_file, lang->data_encoding(), encoding);
     }
 
+    if (dp.key == "CIRCUMFIX") {
+      const char * data = iconv(dp.value);
+      circumfix_flag = *data;
+      // FIXME: Error check, make sure only one character is specified
+    }
+
     /* parse in the flag used by the controlled compound words */
     //else if (d.key == "COMPOUNDFLAG")
     //  compound = data_buf.dup(d.value);
@@ -320,7 +333,7 @@ PosibErr<void> AffixMgr::parse_file(const char * affpath, Conv & iconv)
 
     int numents = 0;      // number of affentry structures to parse
     char achar='\0';      // affix char identifier
-    short xpflg=0;
+    short xpflgs=0;
     AffEntry * nptr;
     {
       // split affix header line into pieces
@@ -337,7 +350,7 @@ PosibErr<void> AffixMgr::parse_file(const char * affpath, Conv & iconv)
       if (dp.key.size != 1 || 
           !(dp.key[0] == 'Y' || dp.key[0] == 'N')) goto error;
       // key is cross product indicator 
-      if (dp.key[0] == 'Y') xpflg = XPRODUCT;
+      if (dp.key[0] == 'Y') xpflgs |= XPRODUCT;
     
       split(dp);
       if (dp.key.empty()) goto error;
@@ -356,7 +369,7 @@ PosibErr<void> AffixMgr::parse_file(const char * affpath, Conv & iconv)
           new (nptr) SfxEntry;
         }
 
-        nptr->xpflg = xpflg;
+        nptr->xpflgs = xpflgs;
 
         split(dp);
         if (dp.key.empty()) goto error;
@@ -381,6 +394,26 @@ PosibErr<void> AffixMgr::parse_file(const char * affpath, Conv & iconv)
         split(dp);
         if (dp.key.empty()) goto error;
         // key is affix string or 0 for null
+
+        // separate out any flags
+        char * f = strchr(dp.key, '/');
+        if (f) {
+          *f = '\0';
+          ++f;
+          dp.key.size = strlen(dp.key.str);
+          nptr->flags = data_buf.dup(iconv(f));
+          two_fold_suffix = true;
+          for (const char * f = nptr->flags; *f; ++f) {
+            if (*f == circumfix_flag)
+              nptr->xpflgs |= CIRCUMFIX;
+          }
+          // FIXME: Add error checking to make sure flags specified
+          // are valid
+        } else {
+          nptr->flags = "";
+        }
+
+        // now handle affix string
         if (dp.key != "0") {
           nptr->appnd  = data_buf.dup(iconv(dp.key));
           nptr->appndl = strlen(nptr->appnd);
@@ -785,13 +818,13 @@ bool AffixMgr::prefix_check (const LookupInfo & linf, ParmString word,
 // check word for suffixes
 bool AffixMgr::suffix_check (const LookupInfo & linf, ParmString word, 
                              IntrCheckInfo & ci, GuessInfo * gi,
-                             int sfxopts, AffEntry * ppfx) const
+                             const PfxEntry * ppfx, const SfxEntry * psfx) const
 {
 
   // first handle the special case of 0 length suffixes
   SfxEntry * se = sStart[0];
   while (se) {
-    if (se->check(linf, word, ci, gi, sfxopts, ppfx)) return true;
+    if (se->check(linf, this, word, ci, gi, ppfx, psfx)) return true;
     se = se->next;
   }
   
@@ -801,7 +834,7 @@ bool AffixMgr::suffix_check (const LookupInfo & linf, ParmString word,
 
   while (sptr) {
     if (isRevSubset(sptr->key(), word + word.size() - 1, word.size())) {
-      if (sptr->check(linf, word, ci, gi, sfxopts, ppfx)) return true;
+      if (sptr->check(linf, this, word, ci, gi, ppfx, psfx)) return true;
       sptr = sptr->next_eq;
     } else {
       sptr = sptr->next_ne;
@@ -838,13 +871,13 @@ bool AffixMgr::affix_check(const LookupInfo & linf, ParmString word,
   if (prefix_check(linf, pword, ci, gi)) return true;
 
   // if still not found check all suffixes
-  if (suffix_check(linf, sword, ci, gi, 0, NULL)) return true;
+  if (suffix_check(linf, sword, ci, gi)) return true;
 
   // if still not found check again but with the lower case version
   // which can make a difference if the entire word matches the cond
   // string
   if (cp == FirstUpper) {
-    return suffix_check(linf, pword, ci, gi, 0, NULL);
+    return suffix_check(linf, pword, ci, gi);
   } else {
     return false;
   }
@@ -859,9 +892,11 @@ void AffixMgr::munch(ParmString word, GuessInfo * gi, bool cross) const
   if (cp == AllUpper) return;
   if (cp != FirstUpper)
     prefix_check(li, word, ci, gi, cross);
-  suffix_check(li, word, ci, gi, 0, NULL);
+  suffix_check(li, word, ci, gi);
 }
 
+// FIXME NOW: expand doesn't work correctly when prefix-suffix
+//            deps. are involved, it basically ignores the prefix
 WordAff * AffixMgr::expand(ParmString word, ParmString aff, 
                            ObjStack & buf, int limit) const
 {
@@ -877,10 +912,7 @@ WordAff * AffixMgr::expand(ParmString word, ParmString aff,
   WordAff * cur = head;
   cur->word = buf.dup(word);
   cur->aff  = suf;
-  {
-    const byte * c = (const byte *)aff.str();
-    const byte * end = c + aff.size();
-    for (;
+  for (const byte *c = (const byte *)aff.str(), *end = c + aff.size();
        c != end; 
        ++c) 
   {
@@ -896,7 +928,6 @@ WordAff * AffixMgr::expand(ParmString word, ParmString aff,
       cur->aff = p->allow_cross() ? csuf : empty;
     }
   }
-  }
   *suf_e = 0;
   *csuf_e = 0;
   cur->next = 0;
@@ -906,14 +937,12 @@ WordAff * AffixMgr::expand(ParmString word, ParmString aff,
   WordAff * * end = &cur->next;
   WordAff * * very_end = end;
   size_t nsuf_s = suf_e - suf + 1;
-  {
   for (WordAff * * cur = &head; cur != end; cur = &(*cur)->next) {
     if ((int)(*cur)->word.size - max_strip_ >= limit) continue;
     byte * nsuf = (byte *)buf.alloc(nsuf_s);
     expand_suffix((*cur)->word, (*cur)->aff, buf, limit, nsuf, &very_end, word);
     (*cur)->aff = nsuf;
   }
-  }
   return head;
 }
 
@@ -940,8 +969,14 @@ WordAff * AffixMgr::expand_suffix(ParmString word, const byte * aff,
         (*cur)->aff  = (const byte *)EMPTY;
         cur = &(*cur)->next;
         expanded = true;
+        // Now handle the two-fold suffix case
+        // FIXME: I am making some, possible invalid, assumtions
+        //        when limit is used
+        if (p->flags[0]) {
+          expand_suffix(newword, (const byte *)p->flags, buf, INT_MAX, 0, &cur, orig_word);
+        }
       }
-    }
+    } 
     if (new_aff && (!expanded || not_expanded)) *new_aff++ = *aff;
     ++aff;
   }
@@ -954,18 +989,14 @@ WordAff * AffixMgr::expand_suffix(ParmString word, const byte * aff,
 CheckAffixRes AffixMgr::check_affix(ParmString word, char aff) const
 {
   CheckAffixRes res = InvalidAffix;
-  {  
   for (PfxEntry * p = pFlag[(unsigned char)aff]; p; p = p->flag_next) {
     res = InapplicableAffix;
     if (p->applicable(word)) return ValidAffix;
   }
-  }
-  {
   for (SfxEntry * p = sFlag[(unsigned char)aff]; p; p = p->flag_next) {
     if (res == InvalidAffix) res = InapplicableAffix;
     if (p->applicable(word)) return ValidAffix;
   }
-  }
   return res;
 }
 
@@ -979,6 +1010,9 @@ CheckAffixRes AffixMgr::check_affix(ParmString word, char aff) const
 int LookupInfo::lookup (ParmString word, const SensitiveCompare * c, 
                         char achar, 
                         WordEntry & o, GuessInfo * gi) const
+  // returns 0 if nothing found
+  // 1 if a match is found
+  // -1 if a word is found but affix doesn't match and "gi"
 {
   SpellerImpl::WS::const_iterator i = begin;
   const char * g = 0;
@@ -1102,31 +1136,35 @@ bool PfxEntry::check(const LookupInfo & linf, const AffixMgr * pmyMgr,
       IntrCheckInfo * guess = 0;
       tmpl += stripl;
 
-      int res = linf.lookup(tmpword, &linf.sp->s_cmp_end, achar, wordinfo, gi);
+      // if a CIRCUMFIX than the prefix is only valid if matched with a suffix
+      // it doesn't make sence if checked it by itself
+      int res = 0;
+      if (!(xpflgs & CIRCUMFIX))
+        res = linf.lookup(tmpword, &linf.sp->s_cmp_end, achar, wordinfo, gi);
 
       if (res == 1) {
 
         lci = &ci;
         lci->word = wordinfo.word;
         goto quit;
-        
+                
       } else if (res == -1) {
 
         guess = gi->head;
 
       }
       
-      // prefix matched but no root word was found 
-      // if XPRODUCT is allowed, try again but now 
-      // cross checked combined with a suffix
+      // prefix matched but no root word was found if XPRODUCT is
+      // allowed, try again but now cross checked combined with a
+      // suffix, also cross check if two_fold_suffix is defined to
+      // allow for prefix-suffix dependencies
       
       if (gi)
         lci = gi->head;
       
-      if (cross && xpflg & XPRODUCT) {
+      if (cross && ((xpflgs & XPRODUCT) || pmyMgr->two_fold_suffix)) {
         if (pmyMgr->suffix_check(linf, ParmString(tmpword, tmpl), 
-                                 ci, gi,
-                                 XPRODUCT, (AffEntry *)this)) {
+                                 ci, gi, this)) {
           lci = &ci;
           
         } else if (gi) {
@@ -1206,21 +1244,33 @@ SimpleString SfxEntry::add(SimpleString word, ObjStack & buf,
 }
 
 // see if this suffix is present in the word 
-bool SfxEntry::check(const LookupInfo & linf, ParmString word,
+bool SfxEntry::check(const LookupInfo & linf, const AffixMgr * pmyMgr,
+                     ParmString word,
                      IntrCheckInfo & ci, GuessInfo * gi,
-                     int optflags, AffEntry* ppfx)
+                     const PfxEntry * ppfx, const SfxEntry * psfx) const
 {
   unsigned              tmpl;		 // length of tmpword 
   int			cond;		 // condition beng examined
   WordEntry             wordinfo;        // hash entry pointer
   byte *	cp;
   VARARRAYM(char, tmpword, word.size()+stripl+1, MAXWORDLEN+1);
-  PfxEntry* ep = (PfxEntry *) ppfx;
 
-  // if this suffix is being cross checked with a prefix
-  // but it does not support cross products skip it
-
-  if ((optflags & XPRODUCT) != 0 &&  (xpflg & XPRODUCT) == 0)
+  // check to make sure if this suffix is even valid in light of:
+  //   1) cross-products, is it allowed by both the prefix and suffix
+  //   2) dependent prefix-sufix is the prefix flag in this->flags
+  //   3) two-stage-suffix is the outer suffix flag in this->flags
+  //   4) the CIRCUMFIX flag
+  bool prefix_suffix_dep = ppfx && TESTAFF(flags, ppfx->achar);
+  // if prefix_suffix_dep no point is doing cross_prod checks
+  bool cross_prod = ppfx && !prefix_suffix_dep && (ppfx->xpflgs & xpflgs & XPRODUCT);
+  if ((ppfx && (!cross_prod && !prefix_suffix_dep)) ||
+      (psfx && !TESTAFF(flags, psfx->achar)) ||
+      (!ppfx && (xpflgs & CIRCUMFIX)) ||
+      // this last test is for compatibility for hunspell, but
+      // shouldn't be necessary since CIRCUMFIX should only be
+      // allowed to be combined via prefix-suffix dependencies
+      // (ie the cross-product flag should not be set)
+      (ppfx && ((ppfx->xpflgs ^ xpflgs) & CIRCUMFIX)))
     return false;
 
   // upon entry suffix is 0 length or already matches the end of the word.
@@ -1260,13 +1310,13 @@ bool SfxEntry::check(const LookupInfo & linf, ParmString word,
       IntrCheckInfo * lci = 0;
       tmpl += stripl;
       const SensitiveCompare * cmp = 
-        optflags & XPRODUCT ? &linf.sp->s_cmp_middle : &linf.sp->s_cmp_begin;
+        ppfx ? &linf.sp->s_cmp_middle : &linf.sp->s_cmp_begin;
       int res = linf.lookup(tmpword, cmp, achar, wordinfo, gi);
-      if (res == 1
-          && ((optflags & XPRODUCT) == 0 || TESTAFF(wordinfo.aff, ep->achar)))
+      if (res == 1 && (!cross_prod ||TESTAFF(wordinfo.aff, ppfx->achar)))
       {
         lci = &ci;
         lci->word = wordinfo.word;
+        goto quit;
       } else if (res == 1 && gi) {
         lci = gi->add();
         lci->word = wordinfo.word;
@@ -1274,11 +1324,32 @@ bool SfxEntry::check(const LookupInfo & linf, ParmString word,
         lci = gi->head;
       }
 
+      // if the root word is not in the dictionary and two_fold_suffix
+      // stripping is enabled, try again with the new root word
+
+      if (pmyMgr->two_fold_suffix && !psfx) {
+        IntrCheckInfo * before = gi ? gi->head : NULL;
+        bool res = (pmyMgr->suffix_check(linf, ParmString(tmpword, tmpl),
+                                         ci, gi, ppfx, this));
+
+        if (gi) {
+          for (IntrCheckInfo * c = gi->head;
+               c != before; 
+               c = const_cast<IntrCheckInfo * >(c->next))
+            add_outer_sfx_info(c);
+        }
+        if (res) {
+          add_outer_sfx_info(&ci);
+          return true;
+        }
+      }
+      
+    quit:
       if (lci) {
         lci->suf_flag = achar;
-        lci->suf_strip_len = stripl;
-        lci->suf_add_len = appndl;
-        lci->suf_add = appnd;
+        lci->inner_suf_strip_len = lci->suf_strip_len = stripl;
+        lci->inner_suf_add_len   = lci->suf_add_len = appndl;
+        lci->inner_suf_add = appnd;
       }
       
       if (lci == &ci) return true;
@@ -1287,6 +1358,16 @@ bool SfxEntry::check(const LookupInfo & linf, ParmString word,
   return false;
 }
 
+void SfxEntry::add_outer_sfx_info(IntrCheckInfo * c) const {
+  c->outer_suf_strip_len = stripl;
+  c->outer_suf_add_len = appndl;
+  c->outer_suf_add = appnd;
+  c->suf_add_len = c->inner_suf_add_len - stripl + appndl;
+  if (c->suf_add_len < 0) c->suf_add_len = 0;
+  int inner_prefix_len = c->suf_add_len - stripl;
+  if (inner_prefix_len < 0) c->suf_strip_len += -inner_prefix_len;
+}
+
 //////////////////////////////////////////////////////////////////////
 //
 // new_affix_mgr
@@ -1340,7 +1421,7 @@ struct AffEntry
    short  stripl;         // length of the strip string
    short  appndl;         // length of the affix string
    short  numconds;       // the number of conditions that must be met
-   short  xpflg;          // flag: XPRODUCT- combine both prefix and suffix 
+   short  xpflgs;          // flag: XPRODUCT- combine both prefix and suffix 
    char   conds[SETSIZE]; // array which encodes the conditions to be met
 };
 
